module CustomerServiceIm
  class ApplicationController < ActionController::API
    include ActionController::ImplicitRender

    def render_500(errors: nil)
      render json: { errors: errors }, status: :internal_server_error
    end

    def render_422(errors: nil)
      render json: { errors: errors }, status: :unprocessable_entity
    end

    def render_401(errors: nil)
      render json: { errors: errors }, status: :unauthorized
    end

    def render_200(data)
      render json: data
    end

    private

    def token_user(user)
      payload = { data: { user_type: user.class.name, user_id: user.id } }
      JWT.encode(payload, hmac_secret, 'HS256')
    end

    def hmac_secret
      CustomerServiceIm.jwt_secret
    end

    def client_has_valid_token?
      !!logged_user
    end

    def logged_user
      begin
        token = request.headers[:token]
        payload, = JWT.decode(token, hmac_secret, true, { algorithm: 'HS256' })
      rescue JWT::DecodeError => e
        # binding.pry
        logger = Logger.new Rails.root.join("log/jwt.log")
        logger.error("JWT Decode Error: #{e}")
        return false
      end

      payload["data"]["user_type"].classify.constantize.find(payload["data"]["user_id"])
    end

    def authenticate_user!
      render json: { error: 'Unauthorized' }, status: :unauthorized if !client_has_valid_token?
    end
  end
end
