
module CustomerServiceIm
  class ConversationsController < ApplicationController
    before_action :authenticate_user!
    before_action :set_conversation, only: [:messages, :all_messages_read]

    def index
      @conversations = Conversation.order('last_messaged_on desc')
      @total_count = @conversations.size
    end

    def show; end

    def create
      @conversation = Conversation.new(user: logged_user)

      if @conversation.save
        render :show
      else
        render_500 errors: @conversation.errors
      end
    end

    def messages
      @messages = @conversation.messages
                               .order('created_at desc')
                               .id_less_then(params[:min_id])
                               .tap do |records|
                                 @total_count = records.size
                               end
                               .page(params[:page] || 1)
                               .per(params[:per] || 8)
                               .reverse
    end

    def all_messages_read
      @conversation.all_messages_read!
      head :no_content
    end

    private

    def set_conversation
      @conversation = Conversation.find(params[:id])
    end
  end
end
