module CustomerServiceIm
  class MessagesController < ApplicationController
    before_action :authenticate_user!
    before_action :set_conversation, only: [ :index ]
    before_action :set_message, only: [ :receive ]

    def index
      @messages = @conversation.messages.order("created_at desc")
    end

    def create
      @message = Message.new(message_params.merge( user: logged_user, receiver_type: "Staff" ))

      if @message.save

        render_200 message: @message
      else
        render_422 errors: @message.errors
      end
    end

    def reply
      @message = Message.new(message_params.merge( staff: logged_user, receiver_type: "User" ))

      if @message.save
        render_200 message: @message
      else
        render_422 errors: @message.errors
      end
    end

    def receive
      if @message.received!
        render_200 message: @message
      else
        render_422 @message.errors
      end
    end

    def read
      if @message.read!
        render_200 message: @message
      else
        render_422 errors: @message.errors
      end
    end

    private

    def message_params
      params.require(:message).permit(
        :conversation_id,
        :content,
        :attachment
      )
    end

    def set_conversation
      @conversation = Conversation.find(message_params[:conversation_id])
    end

    def set_message
      @message = Message.find(params[:id])
    end
  end
end