
module CustomerServiceIm::Concerns::Customer
  extend ActiveSupport::Concern

  def self.included(klass)
    klass.class_eval do
      has_many :conversations, class_name: "CustomerServiceIm::Conversation"
      has_many :messages, class_name: "CustomerServiceIm::Message"
    end

    klass.extend ClassMethods
  end

  def join_conversation(conversation)
    conversation.update(user: self)
  end

  def online!
    redis.sadd online_customers_redis_key, as_redis_record
  end

  def offline!
    redis.sadd online_customers_redis_key, as_redis_record
  end

  # example "User:1"
  def as_redis_record
    "#{self.class.name}/#{self.id}"
  end

  def online?
    id.in? self.class.online_customers_ids
  end

  def offline?
    !online?
  end

  def send_message(content, attachment)
    messages.create(
      conversation: conversation,
      user: self
    )
  end

  def online_customers_redis_key
    self.class.online_customers_redis_key
  end

  def redis
    self.class.redis
  end

  module ClassMethods
    def online_customers
      where(id: online_customers_ids)
    end

    def online_customers_ids
      online_customer_list_in_redis&.map{|str| str.split("/").second.to_i}
    end

    def redis
      @redis ||= Redis.current
    end

    def online_customers_redis_key
      "#{CustomerServiceIm.appid}_online_customers"
    end

    def online_customer_list_in_redis
      redis.smembers online_customers_redis_key
    end

    def find_or_create_mp_user(open_id)
      return if open_id.blank?

      User.create_with(
        email: "#{SecureRandom.uuid}@mp.com",
      ).find_or_create_by(open_id: open_id)
    end
  end
end
