module CustomerServiceIm
  class Conversation < ApplicationRecord
    has_many :messages
    belongs_to :user, polymorphic: true
    belongs_to :staff, optional: true

    after_create :broadcast_to_channel

    # validates_presence_of :title

    def last_message
      messages.last
    end

    def message_created!(message)
      with_lock do
        increment_unread_message_count(message)
        update_last_messaged_on(message)
        broadcast_to_channel

        save!
      end
    end

    def all_messages_read!
      messages.unread.includes(:user).update(state: :read)
      update(staff_unread_message_count: 0)
      broadcast_to_channel
    end

    def broadcast_to_channel
      conversation_data = JSON.parse(ApplicationController.render partial: "customer_service_im/conversations/conversation", locals: { conversation: self })
      ActionCable.server.broadcast "customer_service_im/conversations_channel", conversation_data
    end

    # @param [CustomerServiceIm::Message] message
    def increment_unread_message_count(message)
      if message.from_user?
        increment(:staff_unread_message_count)
      elsif message.from_staff?
        increment(:user_unread_message_count)
      else
        # never arrived
      end
    end

    def update_last_messaged_on(message)
      self.last_messaged_on = message.created_at
    end
  end
end
