module CustomerServiceIm
  class Staff < ApplicationRecord
    has_many :conversations
    has_many :messages

    validates_presence_of :name

    has_secure_password

    def join_conversation(conversation)
      conversation.update(staff: self)

      conversation.messages.from_user.each do |message|
        message.update(staff: self)
        message.broadcast_to_conversation
      end
    end

    def self.online_staffs
      where(id: online_staffs_ids)
    end

    def online!
      redis.sadd self.class.online_staff_redis_key, as_redis_record
    end

    def offline!
      redis.srem self.class.online_staff_redis_key, as_redis_record
    end

    def online?
      id.in? self.class.online_staffs_ids
    end

    def offline?
      !online?
    end

    private

    def self.online_staffs_ids
      online_staff_list_in_redis&.map{|str| str.split("/").second.to_i}
    end

    def self.online_staff_list_in_redis
      redis.smembers online_staff_redis_key
    end

    def self.online_staff_redis_key
      "#{CustomerServiceIm.appid}_online_staffs"
    end

    def self.redis
      @redis ||= Redis.current
    end

    def as_redis_record
      "#{self.class.name}/#{self.id}"
    end

    def redis
      self.class.redis
    end
  end
end
